@@ -1,6 +1,9 @@
 # vim:sw=4:ts=4:et
 # Many thanks to @troopermax <https://github.com/troopermax>
 """Python Ring command line interface."""
+
+from __future__ import annotations
+
 import asyncio
 import functools
 import getpass
@@ -11,28 +14,38 @@ import sys
 import traceback
 from datetime import datetime
 from pathlib import Path, PurePath
+from typing import Sequence, cast
 
 import asyncclick as click
 
-from ring_doorbell import Auth, AuthenticationError, Requires2FAError, Ring, RingEvent
-from ring_doorbell.const import CLI_TOKEN_FILE, PACKAGE_NAME, USER_AGENT
+from ring_doorbell import (
+    Auth,
+    AuthenticationError,
+    Requires2FAError,
+    Ring,
+    RingDoorBell,
+    RingEvent,
+    RingGeneric,
+)
+from ring_doorbell.const import CLI_TOKEN_FILE, GCM_TOKEN_FILE, PACKAGE_NAME, USER_AGENT
 from ring_doorbell.listen import can_listen
 
 
-def _header():
+def _header() -> None:
     _bar()
     echo("Ring CLI")
 
 
-def _bar():
+def _bar() -> None:
     echo("---------------------------------")
 
 
-click.anyio_backend = "asyncio"
+click.anyio_backend = "asyncio"  # type: ignore[attr-defined]
 
 pass_ring = click.make_pass_decorator(Ring)
 
 cache_file = Path(CLI_TOKEN_FILE)
+gcm_cache_file = Path(GCM_TOKEN_FILE)
 
 
 class ExceptionHandlerGroup(click.Group):
@@ -57,7 +70,7 @@ class ExceptionHandlerGroup(click.Group):
 class MutuallyExclusiveOption(click.Option):
     """Prevents incompatable options being supplied, i.e. on and off."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         self.mutually_exclusive = set(kwargs.pop("mutually_exclusive", []))
         _help = kwargs.get("help", "")
         if self.mutually_exclusive:
@@ -70,10 +83,11 @@ class MutuallyExclusiveOption(click.Option):
 
     async def handle_parse_result(self, ctx, opts, args):
         if self.mutually_exclusive.intersection(opts) and self.name in opts:
-            raise click.UsageError(
+            msg = (
                 "Illegal usage: `{}` is mutually exclusive with "
                 "arguments `{}`.".format(self.name, ", ".join(self.mutually_exclusive))
             )
+            raise click.UsageError(msg)
 
         return await super().handle_parse_result(ctx, opts, args)
 
@@ -81,8 +95,8 @@ class MutuallyExclusiveOption(click.Option):
 echo = click.echo
 
 
-def token_updated(token):
-    """Writes token to file"""
+def token_updated(token) -> None:
+    """Writes token to file."""
     cache_file.write_text(json.dumps(token), encoding="utf-8")
 
 
@@ -96,8 +110,7 @@ def _format_filename(device_name, event):
         device_name, event["created_at"], event["kind"], answered_status, event["id"]
     )
 
-    filename = filename.replace(" ", "_").replace(":", ".") + ".mp4"
-    return filename
+    return filename.replace(" ", "_").replace(":", ".") + ".mp4"
 
 
 def _do_auth(username, password, user_agent=USER_AGENT):
@@ -118,9 +131,10 @@ def _do_auth(username, password, user_agent=USER_AGENT):
 
 def _get_ring(username, password, do_update_data, user_agent=USER_AGENT):
     # connect to Ring account
-    global cache_file
+    global cache_file, gcm_cache_file
     if user_agent != USER_AGENT:
         cache_file = Path(user_agent + ".token.cache")
+        gcm_cache_file = Path(user_agent + ".gcm_token.cache")
     if cache_file.is_file():
         auth = Auth(
             user_agent,
@@ -193,26 +207,24 @@ async def cli(ctx, username, password, debug, user_agent):
 
     if ctx.invoked_subcommand is None:
         return await ctx.invoke(show)
+    return None
 
 
 @cli.command(name="list")
 @pass_ring
-async def list_command(ring: Ring):
+async def list_command(ring: Ring) -> None:
     """List ring devices."""
     devices = ring.devices()
-
-    doorbells = devices["doorbots"]
-    chimes = devices["chimes"]
-    stickup_cams = devices["stickup_cams"]
-    other = devices["other"]
-
-    for device in doorbells:
+    device: RingGeneric | None = None
+    for device in devices.doorbots:
         echo(device)
-    for device in chimes:
+    for device in devices.authorized_doorbots:
         echo(device)
-    for device in stickup_cams:
+    for device in devices.chimes:
         echo(device)
-    for device in other:
+    for device in devices.stickup_cams:
+        echo(device)
+    for device in devices.other:
         echo(device)
 
 
@@ -255,22 +267,22 @@ async def motion_detection(ctx, ring: Ring, device_name, turn_on, turn_off):
         )
         return await ctx.invoke(list_command)
     if not device.has_capability("motion_detection"):
-        echo(f"{str(device)} is not capable of motion detection")
-        return
-
+        echo(f"{device!s} is not capable of motion detection")
+        return None
+    device = cast(RingDoorBell, device)
     state = "on" if device.motion_detection else "off"
     if not turn_on and not turn_off:
-        echo(f"{str(device)} has motion detection {state}")
-        return
+        echo(f"{device!s} has motion detection {state}")
+        return None
     is_on = device.motion_detection
     if (turn_on and is_on) or (turn_off and not is_on):
-        echo(f"{str(device)} already has motion detection {state}")
-        return
+        echo(f"{device!s} already has motion detection {state}")
+        return None
 
     device.motion_detection = turn_on if turn_on else False
     state = "on" if device.motion_detection else "off"
-    echo(f"{str(device)} motion detection set to {state}")
-    return
+    echo(f"{device!s} motion detection set to {state}")
+    return None
 
 
 @cli.command()
@@ -285,7 +297,7 @@ async def motion_detection(ctx, ring: Ring, device_name, turn_on, turn_off):
 @click.pass_context
 async def show(ctx, ring: Ring, device_name):
     """Display ring devices."""
-    devices = None
+    devices: Sequence[RingGeneric] | None = None
 
     if device_name and (device := ring.get_device_by_name(device_name)):
         devices = [device]
@@ -307,6 +319,7 @@ async def show(ctx, ring: Ring, device_name):
         echo("Wifi Name:  %s" % dev.wifi_name)
         echo("Wifi RSSI:  %s" % dev.wifi_signal_strength)
         echo()
+    return None
 
 
 @cli.command(name="devices")
@@ -345,12 +358,14 @@ async def devices_command(ctx, ring: Ring, device_name, json_flag):
 
     if device_json:
         echo(json.dumps(device_json, indent=2))
+        return None
     else:
         for device_type in ring.devices_data:
             for device_api_id in ring.devices_data[device_type]:
                 echo(
                     json.dumps(ring.devices_data[device_type][device_api_id], indent=2)
                 )
+        return None
 
 
 @cli.command()
@@ -362,7 +377,7 @@ async def devices_command(ctx, ring: Ring, device_name, json_flag):
     help="Output raw json",
 )
 @pass_ring
-async def dings(ring: Ring, json_flag):
+async def dings(ring: Ring, json_flag) -> None:
     """Get dings information."""
     if not json_flag:
         echo(
@@ -381,7 +396,7 @@ async def dings(ring: Ring, json_flag):
     help="Output raw json",
 )
 @pass_ring
-async def groups(ring: Ring, json_flag):
+async def groups(ring: Ring, json_flag) -> None:
     """Get group information."""
     if not json_flag:
         echo(
@@ -391,8 +406,10 @@ async def groups(ring: Ring, json_flag):
     if not ring.groups_data:
         echo("No ring device groups setup")
     else:
-        for group in ring.groups_data:
-            echo(json.dumps(group, indent=2))
+        for light_group in ring.groups().values():
+            light_group.update()
+            echo(json.dumps(light_group._attrs, indent=2))
+            echo(json.dumps(light_group._health_attrs, indent=2))
 
 
 @cli.command()
@@ -403,7 +420,7 @@ async def groups(ring: Ring, json_flag):
     help="Url to query, i.e. /clients_api/dings/active",
 )
 @pass_ring
-async def raw_query(ring: Ring, url):
+async def raw_query(ring: Ring, url) -> None:
     """Directly query a url and return json result."""
     data = ring.query(url).json()
     echo(json.dumps(data, indent=2))
@@ -427,7 +444,7 @@ async def raw_query(ring: Ring, url):
     "--kind",
     required=False,
     default=None,
-    type=click.Choice(["ding", "motion"], case_sensitive=False),
+    type=click.Choice(["ding", "motion", "on_demand"], case_sensitive=False),
     help="Get devices",
 )
 @click.option(
@@ -456,6 +473,7 @@ async def history_command(ctx, ring: Ring, device_name, kind, limit, json_flag):
 
     history = device.history(limit=limit, kind=kind, convert_timezone=False)
     echo(json.dumps(history, indent=2))
+    return None
 
 
 @cli.command()
@@ -506,7 +524,7 @@ async def videos(
 ):
     """Interact with ring videos."""
     device = None
-    if device_name and not (device := ring.get_device_by_name(device_name)):
+    if device_name and not (device := ring.get_video_device_by_name(device_name)):
         echo(
             f"No device with name {device_name} found. "
             + "List of found device names (kind) is:"
@@ -514,7 +532,7 @@ async def videos(
         return await ctx.invoke(list_command)
     if device and not device.has_capability("video"):
         echo(f"Device {device.name} is not a video device")
-        return
+        return None
     # return the first device is implemented to be consistent with previous cli version
     if not device:
         if video_devices := ring.video_devices():
@@ -526,9 +544,17 @@ async def videos(
             )
             return await ctx.invoke(list_command)
 
-    if not count and not download and not download_all:
-        echo("Last recording url is: " + device.recording_url(device.last_recording_id))
-        return
+    if not device:  # Make mypy happy
+        return None
+    if (
+        not count
+        and not download
+        and not download_all
+        and device.last_recording_id
+        and (url := device.recording_url(device.last_recording_id))
+    ):
+        echo("Last recording url is: " + url)
+        return None
 
     events = None
     if download_all:
@@ -567,26 +593,26 @@ async def videos(
         if events is None:
             echo(
                 "\tGetting videos linked on your Ring account.\n"
-                + "\tThis may take some time....\n"
+                "\tThis may take some time....\n"
             )
             events = _get_events(device, max_count)
 
         echo(
             f"\tDownloading {len(events)} videos linked on your Ring account.\n"
-            + "\tThis may take some time....\n"
+            "\tThis may take some time....\n"
         )
-        counter = 0
-        for event in events:
-            counter += 1
+        for counter, event in enumerate(events):
             filename = str(PurePath(download_to, _format_filename(device.name, event)))
             echo(f"\t{counter}/{len(events)} Downloading {filename}")
 
             device.recording_download(event["id"], filename=filename, override=False)
+        return None
+    return None
 
 
-async def ainput(string: str) -> str:
+async def ainput(string: str):
     loop = asyncio.get_event_loop()
-    await loop.run_in_executor(None, lambda s=string: sys.stdout.write(s + " "))
+    await loop.run_in_executor(None, lambda s=string: sys.stdout.write(s + " "))  # type: ignore[misc]
 
     def read_with_timeout(timeout):
         if select.select(
@@ -598,8 +624,7 @@ async def ainput(string: str) -> str:
             timeout,
         )[0]:
             # line = sys.stdin.next()
-            line = sys.stdin.readline()
-            return line
+            return sys.stdin.readline()
         return None
 
     line = None
@@ -613,10 +638,10 @@ def get_now_str():
 
 
 class _event_handler:  # pylint:disable=invalid-name
-    def __init__(self, ring: Ring):
+    def __init__(self, ring: Ring) -> None:
         self.ring = ring
 
-    def on_event(self, event: RingEvent):
+    def on_event(self, event: RingEvent) -> None:
         msg = (
             get_now_str()
             + ": "
@@ -631,7 +656,7 @@ class _event_handler:  # pylint:disable=invalid-name
 @click.option(
     "--credentials-file",
     required=False,
-    default="credentials.json",
+    default=None,
     help=(
         "File to store push credentials, "
         + "if not provided credentials will be recreated from scratch"
@@ -639,14 +664,14 @@ class _event_handler:  # pylint:disable=invalid-name
 )
 @click.option(
     "--store-credentials/--no-store-credentials",
-    default=False,
+    default=True,
     help="Whether or not to store the push credentials, default is false",
 )
 @click.option(
     "--show-credentials",
     default=False,
     is_flag=True,
-    help="Whether or not to store the push credentials, default is false",
+    help="Whether or not to show the push credentials, default is false",
 )
 @pass_ring
 @click.pass_context
@@ -656,7 +681,7 @@ async def listen(
     store_credentials,
     credentials_file,
     show_credentials,
-):
+) -> None:
     """Listen to push notification like the ones sent to your phone."""
     if not can_listen:
         echo("Ring is not configured for listening to notifications!")
@@ -667,7 +692,7 @@ async def listen(
         RingEventListener,
     )
 
-    def credentials_updated_callback(credentials):
+    def credentials_updated_callback(credentials) -> None:
         if store_credentials:
             with open(credentials_file, "w", encoding="utf-8") as f:
                 json.dump(credentials, f)
@@ -676,8 +701,13 @@ async def listen(
             if show_credentials:
                 echo(credentials)
 
+    if not credentials_file:
+        credentials_file = gcm_cache_file
+    else:
+        credentials_file = Path(credentials_file)
+
     credentials = None
-    if store_credentials and Path(credentials_file).is_file():
+    if store_credentials and credentials_file.is_file():
         # already registered, load previous credentials
         with open(credentials_file, encoding="utf-8") as f:
             credentials = json.load(f)
